package com.spacester.tweetsterupdate.emailAuth;

import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;
import com.muddzdev.styleabletoast.StyleableToast;
import com.spacester.tweetsterupdate.MainActivity;
import com.spacester.tweetsterupdate.R;
import com.spacester.tweetsterupdate.activity.Policy;
import com.spacester.tweetsterupdate.activity.Terms;

import java.util.HashMap;
import java.util.Objects;

@SuppressWarnings("ALL")
public class SignUpActivity extends AppCompatActivity {

    FirebaseAuth mAuth;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_sign_up);

         mAuth = FirebaseAuth.getInstance();

        //Progress
        ProgressBar progressBar = findViewById(R.id.pb);

        //Back
        ImageView back = findViewById(R.id.imageView);
        back.setOnClickListener(v -> onBackPressed());

        //Get Email
        String email = getIntent().getStringExtra("email").trim();

        TextView privacy = findViewById(R.id.privacy);
        TextView terms = findViewById(R.id.terms);

        privacy.setOnClickListener(v -> {
            Intent intent = new Intent(getApplicationContext(), Policy.class);
            startActivity(intent);
        });
        terms.setOnClickListener(v -> {
            Intent intent = new Intent(getApplicationContext(), Terms.class);
            startActivity(intent);
        });

        //SignUp
        EditText name = findViewById(R.id.editText);
        EditText username = findViewById(R.id.username);
        EditText password = findViewById(R.id.password);

        Button button = findViewById(R.id.signUp);

        button.setOnClickListener(v -> {
            String mName = name.getText().toString().trim();
            String mUsername = username.getText().toString().trim();
            String mPassword = password.getText().toString().trim();
            progressBar.setVisibility(View.VISIBLE);
            if (mName.isEmpty()){
                progressBar.setVisibility(View.GONE);
                Toast.makeText(this, "Enter Your Name", Toast.LENGTH_SHORT).show();

            }else if (mUsername.isEmpty()){
                progressBar.setVisibility(View.GONE);

                Toast.makeText(this, "Enter Your Username", Toast.LENGTH_SHORT).show();

            }else if (mPassword.isEmpty()){
                progressBar.setVisibility(View.GONE);

                Toast.makeText(this, "Enter Your Password", Toast.LENGTH_SHORT).show();

            }else if (mPassword.length() < 6) {
                progressBar.setVisibility(View.GONE);

                Toast.makeText(this, "Your Password should contain minimum 6 characters", Toast.LENGTH_SHORT).show();

            }else {

                //Check if username exist
                FirebaseDatabase.getInstance().getReference("Users").addListenerForSingleValueEvent(new ValueEventListener() {
                    @Override
                    public void onDataChange(@NonNull DataSnapshot snapshot) {
                        if (snapshot.exists()){
                            Query query = FirebaseDatabase.getInstance().getReference("Users").orderByChild("username").equalTo(mUsername);
                            query.addListenerForSingleValueEvent(new ValueEventListener() {
                                @Override
                                public void onDataChange(@NonNull DataSnapshot snapshot) {
                                    if (snapshot.getChildrenCount()>0){
                                        progressBar.setVisibility(View.GONE);
                                        new StyleableToast
                                                .Builder(getApplicationContext())
                                                .text("Username already exist")
                                                .textColor(Color.WHITE)
                                                .textBold()
                                                .gravity(0)
                                                .length(2000)
                                                .solidBackground()
                                                .backgroundColor(getResources().getColor(R.color.colorPrimary))
                                                .show();
                                    }else {
                                        mAuth.createUserWithEmailAndPassword(email,mPassword).addOnCompleteListener(task -> {
                                            if (task.isSuccessful()) {

                                                FirebaseUser firebaseUser = mAuth.getCurrentUser();
                                                String userId = Objects.requireNonNull(firebaseUser).getUid();

                                                DatabaseReference reference = FirebaseDatabase.getInstance().getReference().child("Users").child(userId);

                                                HashMap<String, Object> hashMap = new HashMap<>();
                                                hashMap.put("id", userId);
                                                hashMap.put("name", mName);
                                                hashMap.put("email", email);
                                                hashMap.put("username", mUsername);
                                                hashMap.put("bio", "");
                                                hashMap.put("verified", "");
                                                hashMap.put("location", "");
                                                hashMap.put("status", "");
                                                hashMap.put("phone", "");
                                                hashMap.put("typingTo", "noOne");
                                                hashMap.put("link", "");
                                                hashMap.put("cover", "");
                                                hashMap.put("photo", "");

                                                reference.setValue(hashMap).addOnCompleteListener(task1 -> {

                                                    if (task1.isSuccessful()) {
                                                        progressBar.setVisibility(View.GONE);
                                                        Intent intent = new Intent(SignUpActivity.this, MainActivity.class);
                                                        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
                                                        startActivity(intent);
                                                        finish();

                                                    }

                                                });

                                            } else {
                                                progressBar.setVisibility(View.GONE);
                                                String msg = Objects.requireNonNull(task.getException()).getMessage();
                                                Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_LONG).show();
                                            }
                                        });
                                    }
                                }

                                @Override
                                public void onCancelled(@NonNull DatabaseError error) {
                                    progressBar.setVisibility(View.GONE);

                                    Toast.makeText(getApplicationContext(), error.getMessage(), Toast.LENGTH_LONG).show();
                                }
                            });
                        }else {
                            mAuth.createUserWithEmailAndPassword(email,mPassword).addOnCompleteListener(task -> {
                                if (task.isSuccessful()) {

                                    FirebaseUser firebaseUser = mAuth.getCurrentUser();
                                    String userId = Objects.requireNonNull(firebaseUser).getUid();

                                    DatabaseReference reference = FirebaseDatabase.getInstance().getReference().child("Users").child(userId);

                                    HashMap<String, Object> hashMap = new HashMap<>();
                                    hashMap.put("id", userId);
                                    hashMap.put("name", mName);
                                    hashMap.put("email", email);
                                    hashMap.put("username", mUsername);
                                    hashMap.put("bio", "");
                                    hashMap.put("verified", "");
                                    hashMap.put("location", "");
                                    hashMap.put("status", "");
                                    hashMap.put("phone", "");
                                    hashMap.put("typingTo", "noOne");
                                    hashMap.put("link", "");
                                    hashMap.put("cover", "");
                                    hashMap.put("photo", "");

                                    reference.setValue(hashMap).addOnCompleteListener(task1 -> {

                                        if (task1.isSuccessful()) {
                                            progressBar.setVisibility(View.GONE);
                                            Intent intent = new Intent(SignUpActivity.this, MainActivity.class);
                                            intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
                                            startActivity(intent);
                                            finish();

                                        }

                                    });

                                } else {
                                    progressBar.setVisibility(View.GONE);
                                    String msg = Objects.requireNonNull(task.getException()).getMessage();
                                    Toast.makeText(SignUpActivity.this, msg, Toast.LENGTH_LONG).show();
                                }
                            });
                        }
                    }

                    @Override
                    public void onCancelled(@NonNull DatabaseError error) {
                        progressBar.setVisibility(View.GONE);

                        Toast.makeText(SignUpActivity.this, error.getMessage(), Toast.LENGTH_LONG).show();
                    }
                });


            }
        });

    }

    @Override
    protected void onStart() {
        super.onStart();
        FirebaseUser currentUser = mAuth.getCurrentUser();
        if (currentUser == null){
        }else {
           onBackPressed();
        }
    }
}